<?php

namespace Drupal\ino_complementary_content\Plugin\Block;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\ino_complementary_content\InoCustomBlock;
use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;
use Drupal\site_settings\Entity\SiteSettingEntity;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Class InoFooter.
 *
 * @Block(
 *   id = "sidebar_block",
 *   admin_label= @Translation("Custom sidebar"),
 *   category = @Translation("Custom blocks")
 * )
 *
 * @package Drupal\ino_complementary_content\Plugin\Block
 */
class InoSidebar extends InoCustomBlock {

  /**
   * The current request object.
   *
   * @var null|\Symfony\Component\HttpFoundation\Request
   */
  protected $currentRequest;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entityTypeManager, LanguageManagerInterface $languageManager, RequestStack $requestStack) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entityTypeManager, $languageManager);
    $this->currentRequest = $requestStack->getCurrentRequest();

    $side = $this->configuration['block_side'] ?: 'left';
    $this->siteSettingsType = 'ino_ss_sidebar_' . $side;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('language_manager'),
      $container->get('request_stack')
    );
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   *   Thrown if the entity type doesn't exist.
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   *   Thrown if the storage handler couldn't be loaded.
   */
  public function build() {
    $build = [];
    $side = $this->configuration['block_side'] ?: 'left';

    /** @var \Drupal\site_settings\Entity\SiteSettingEntity $header_settings */
    $sidebar_settings = $this->loadSiteSettings();
    $output = $this->getParagraphs($sidebar_settings);

    if (empty($output)) {
      return $build;
    }

    $build = [
      '#theme' => "ino_sidebar_$side",
      '#paragraphs' => $output,
    ];

    return $build;
  }

  /**
   * Loads the paragraphs from the configuration page.
   *
   * @param null|\Drupal\site_settings\Entity\SiteSettingEntity $settings
   *   The source site_settings entity.
   *
   * @return array
   *   The paragraphs render array.
   */
  protected function getParagraphs($settings) {
    $paragraphs = [];

    if (NULL !== $settings) {
      $config_paragraphs = $this->getSettingParagraphs($settings);
      if (is_array($config_paragraphs)) {
        $paragraphs = array_merge($paragraphs, $config_paragraphs);
      }
    }

    $currentNode = $this->currentRequest->get('node');
    if ($currentNode instanceof Node && $currentNode->hasField('field_n_sidebar_' . $this->configuration['block_side'])) {
      $paragraphs = array_merge($paragraphs, $this->getNodeParagraphs($currentNode));
    }

    $view_builder = $this->entityTypeManager->getViewBuilder('paragraph');
    if (!empty($paragraphs)) {
      return $view_builder->viewMultiple($paragraphs ?: []);
    }

    return [];
  }

  /**
   * Loads the paragraphs from the current node.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node object.
   *
   * @return array
   *   The paragraphs render array.
   */
  protected function getNodeParagraphs(NodeInterface $node) {
    return $node->get('field_n_sidebar_' . $this->configuration['block_side'])->referencedEntities();
  }

  /**
   * Loads the paragraphs from the configuration page.
   *
   * @param null|\Drupal\site_settings\Entity\SiteSettingEntity $settings
   *   The source site_settings entity.
   *
   * @return array
   *   The paragraphs render array.
   */
  protected function getSettingParagraphs($settings) {
    if (!($settings instanceof SiteSettingEntity)) {
      return NULL;
    }

    $paragraphs = $settings->get('field_ss_sidebar_' . $this->configuration['block_side'])->referencedEntities();
    if (empty($paragraphs)) {
      return NULL;
    }

    return $paragraphs;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $default_value = $this->configuration['block_side'] ?? 'left';
    $form['block_side'] = [
      '#title' => $this->t('Side'),
      '#description' => $this->t('Which side should the sidebar appear on?'),
      '#type' => 'select',
      '#options' => [
        'left' => $this->t('Left'),
        'right' => $this->t('Right'),
      ],
      '#default_value' => $default_value,
      '#required' => TRUE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    $this->configuration['block_side'] = $form_state->getValue('block_side');
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   *   Thrown if the entity type doesn't exist.
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   *   Thrown if the storage handler couldn't be loaded.
   */
  public function getCacheTags() {
    /** @var \Drupal\node\Entity\Node $currentNode */
    $currentNode = $this->currentRequest->get('node');
    if ($currentNode instanceof Node && $currentNode->hasField('field_n_sidebar_' . $this->configuration['block_side'])) {
      return Cache::mergeTags(parent::getCacheTags(), $currentNode->getCacheTags());
    }

    return parent::getCacheTags();
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    $side = $this->configuration['block_side'] ?: 'left';

    return Cache::mergeContexts(
      parent::getCacheContexts(),
      ['route.ino_cc_node:field_n_sidebar_' . $side]
    );
  }

}
