<?php

namespace Drupal\ino_homepage\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Controller routines for user routes.
 */
class HomepageController extends ControllerBase {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The language manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, LanguageManagerInterface $languageManager, RequestStack $requestStack) {
    $this->entityTypeManager = $entityTypeManager;
    $this->languageManager = $languageManager;
    $this->requestStack = $requestStack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('language_manager'),
      $container->get('request_stack')
    );
  }

  /**
   * Homepage page content.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   *   Thrown if the entity type doesn't exist.
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   *   Thrown if the storage handler couldn't be loaded.
   */
  public function homePage() {
    $uri = trim($this->requestStack->getCurrentRequest()->getRequestUri(), '/');
    $current_lang = $this->languageManager->getCurrentLanguage()->getId();

    // Redirect to frontpage:
    if ($uri && ($uri != $current_lang)) {
      $url = Url::fromRoute('<front>');
      $response = new RedirectResponse($url->toString());
      $response->send();
    }

    // Show site settings entity:
    $build = [];
    /** @var \Drupal\site_settings\Entity\SiteSettingEntity $homepage */
    $homepage = $this->loadSiteSettings('ino_ss_homepage');
    if (NULL !== $homepage) {
      /** @var \Drupal\Core\Entity\EntityViewBuilder $render */
      $render = $this->entityTypeManager
        ->getViewBuilder($homepage->getEntityTypeId());

      $build = $render->view($homepage, 'default');

      // Add site_setting context menu.
      $build['#contextual_links'] = [
        'site_settings' => [
          'route_parameters' => ['site_setting_entity' => $homepage->id()],
        ],
      ];
    }

    $build['#cache']['tags'][] = '';
    return $build;
  }

  /**
   * Loads the given configuration page, and its fallback.
   *
   * @param string $site_setting_type
   *   The configuration page id.
   *
   * @return null|\Drupal\site_settings\Entity\SiteSettingEntity
   *   The site_settings entity.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   *   Thrown if the entity type doesn't exist.
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   *   Thrown if the storage handler couldn't be loaded.
   */
  protected function loadSiteSettings($site_setting_type) {
    $current_langcode = $this->languageManager->getCurrentLanguage()->getId();

    $entities = $this->entityTypeManager
      ->getStorage('site_setting_entity')
      ->loadByProperties(['type' => $site_setting_type]);

    if (empty($entities)) {
      return NULL;
    }

    /** @var \Drupal\site_settings\Entity\SiteSettingEntity $settings */
    $settings = array_shift($entities);

    if ($settings->hasTranslation($current_langcode)) {
      return $settings->getTranslation($current_langcode);
    }

    return $settings;
  }

}
