<?php

namespace Drupal\ino_pt_list_term_reference_filter\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\TypedData\MapDataDefinition;

/**
 * Class TermReferenceFilterField.
 *
 * @FieldType(
 *   id = "ino_list_term_reference_filter",
 *   label = @Translation("Term translation filter"),
 *   description = @Translation("Stores vocabulary-based taxonomy terms."),
 *   default_widget = "ino_list_term_reference_filter",
 *   default_formatter= "ino_list_term_reference_filter"
 * )
 */
class TermReferenceFilterField extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    return [
      'settings' => MapDataDefinition::create()
        ->setLabel(new TranslatableMarkup('Options'))
        ->setClass('\Drupal\ino_pt_list\TermReferenceFilter'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'settings' => [
          'type' => 'blob',
          'size' => 'normal',
          'not null' => TRUE,
          'serialize' => TRUE,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    return empty($this->get('settings')->getValue());
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultFieldSettings() {
    $r = [
      'trfw_node_type_field' => '',
      'trfw_taxonomy_type_field' => '',
    ] + parent::defaultFieldSettings();

    return $r;
  }

  /**
   * {@inheritdoc}
   */
  public function fieldSettingsForm(array $form, FormStateInterface $form_state) {
    $fields = $this->getEntity()->getFieldDefinitions();

    $options = [];
    foreach ($fields as $field_name => $field_info) {
      /** @var \Drupal\field\Entity\FieldConfig $fieldItem */
      if ('entity_reference' == $field_info->getType() && is_a($field_info, 'Drupal\field\Entity\FieldConfig')) {
        $options[$field_name] = $field_info->getLabel();
      }
    }

    $elements = parent::fieldSettingsForm($form, $form_state);

    $elements['trfw_node_type_field'] = [
      '#type' => 'select',
      '#required' => FALSE,
      '#title' => $this->t('Node type field'),
      '#default_value' => $this->getSetting('trfw_node_type_field'),
      '#empty_option' => $this->t('- Select -'),
      '#options' => $options,
    ];

    $elements['trfw_taxonomy_type_field'] = [
      '#type' => 'select',
      '#required' => FALSE,
      '#title' => $this->t('Taxonomy type field'),
      '#default_value' => $this->getSetting('trfw_taxonomy_type_field'),
      '#empty_option' => $this->t('- Select -'),
      '#options' => $options,
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function preSave() {
    parent::preSave();

    $node_type_field = $this->getSetting('trfw_node_type_field');
    $taxonomy_type_field = $this->getSetting('trfw_taxonomy_type_field');
    $entity = $this->getEntity();

    if (!empty($node_type_field) && !empty($taxonomy_type_field) && $entity->hasField($node_type_field)) {
      $selected_type_values = $entity->get($node_type_field)->getValue();
      $selected_types = [];
      foreach ($selected_type_values as $type_value) {
        $selected_types[$type_value['target_id']] = $type_value['target_id'];
      }

      $node_type_vocabs = $this->getNodeTypeVocabularies($selected_types);
      $values = $this->values;
      foreach ($values['settings'] as $vocab => $value) {
        foreach ($selected_types as $selected_type) {
          if (!isset($node_type_vocabs[$selected_type][$vocab])) {
            unset($values['settings'][$vocab]);
          }
        }
      }

      $this->set('settings', $values['settings']);
    }
  }

  /**
   * Get node type vocabulary in reference field.
   *
   * @param array $node_type_ids
   *   Node type list.
   *
   * @return array
   *   Return vocabularies and node type array.
   */
  protected function getNodeTypeVocabularies(array $node_type_ids) {
    $entity_field_manager = \Drupal::service('entity_field.manager');
    $node_type_vocab = [];

    foreach ($node_type_ids as $node_type) {
      if ('_none' != $node_type) {
        $node_type_vocab[$node_type] = [];
        $fields = $entity_field_manager->getFieldDefinitions('node', $node_type);
        foreach ($fields as $field_item) {
          /** @var \Drupal\field\Entity\FieldConfig $field_item */
          if ('entity_reference' == $field_item->getType()) {
            $settings = $field_item->getSettings();
            if (isset($settings['target_type']) && 'taxonomy_term' == $settings['target_type']) {
              $node_type_vocab[$node_type] += $settings['handler_settings']['target_bundles'];
            }
          }
        }
      }
    }

    return $node_type_vocab;
  }

}
