<?php

/**
 * @file
 * Contains \Drupal\ino_pt_social_post\Plugin\Field\FieldFormatter\SocialPostFormatter.
 */

namespace Drupal\ino_pt_social_post\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Unicode;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Url;
use Drupal\ino_pt_social_post\Factory\SocialPostFormatterFactoryInterface;
use Drupal\language\Entity\ConfigurableLanguage;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\Exception\InvalidArgumentException;

/**
 * Plugin implementation of the 'social post' formatter.
 *
 * @FieldFormatter (
 *   id = "social-post",
 *   label = @Translation("Social post"),
 *   field_types = {
 *     "social-post"
 *   }
 * )
 */
class SocialPostFormatter extends FormatterBase {
  use LoggerChannelTrait;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The social post formatter factory.
   *
   * @var \Drupal\ino_pt_social_post\Factory\SocialPostFormatterFactoryInterface
   */
  protected $socialPostFormatter;

  /**
   * Constructs an InoPtSocialPost object.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager service.
   * @param \Drupal\ino_pt_social_post\Factory\SocialPostFormatterFactoryInterface $social_post_formatter
   *   Social post formatter factory.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, LanguageManagerInterface $language_manager, SocialPostFormatterFactoryInterface $social_post_formatter) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);

    $this->fieldDefinition = $field_definition;
    $this->settings = $settings;
    $this->label = $label;
    $this->viewMode = $view_mode;
    $this->thirdPartySettings = $third_party_settings;
    $this->languageManager = $language_manager;
    $this->socialPostFormatter = $social_post_formatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('language_manager'),
      $container->get('ino_pt_social_post.social_post_formatter_factory')
    );
  }

  /**
   * Gets the logger for a specific channel.
   *
   * This method exists for backward-compatibility between FormBase and
   * LoggerChannelTrait. Use LoggerChannelTrait::getLogger() instead.
   *
   * @param string $channel
   *   The name of the channel. Can be any string, but the general practice is
   *   to use the name of the subsystem calling this.
   *
   * @return \Psr\Log\LoggerInterface
   *   The logger for the given channel.
   */
  protected function logger($channel) {
    return $this->getLogger($channel);
  }

  /**
   * {@inheritdoc}
   *
   * Social posts without extra wrappers for each network (field__item).
   */
  public function view(FieldItemListInterface $items, $langcode = NULL) {
    // Default the language to the current content language.
    if (empty($langcode)) {
      $langcode = $this->languageManager->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)->getId();
    }
    $post_groups = $this->viewElements($items, $langcode);
    $networks = [];
    $language_selectors = [];
    foreach ($post_groups as $post_group) {
      $social_network = $post_group['#ino_pt_social_posts_info']['network'];
      $networks[] = $social_network;
      if (count($post_group['#ino_pt_social_posts_info']['langcodes']) > 1) {
        // Add a language selector for this social network.
        // Requires the dropdown_language module.
        $links = [];
        foreach ($post_group['#ino_pt_social_posts_info']['langcodes'] as $post_langcode) {
          $url = Url::fromUserInput('#' . $post_langcode);
          $links[$post_langcode] = [
            'url' => $url,
            'title' => Unicode::ucfirst($post_langcode),
            'language' => ConfigurableLanguage::createFromLangcode($post_langcode),
            'attributes' => [
              'class' => ['language-link'],
            ],
            'query' => [],
          ];
          if ($post_langcode == $langcode) {
            $links[$post_langcode]['attributes']['class'][] = 'is-active';
          }
        }
        $language_selectors[$social_network] = [
          '#theme_wrappers' => ['container'],
          '#attributes' => [
            // The dropdown needs this block id to work properly.
            'class' => ['block-language'],
            'data-network' => $social_network,
          ],
          'content' => [
            '#theme' => 'links',
            '#links' => $links,
            '#attributes' => [
              'class' => [
                "sp-language-item",
              ],
            ],
          ],
        ];
      }
    }
    // Add controls to the top.
    array_unshift($post_groups, [
      '#theme' => 'ino_pt_social_post_selectors',
      '#networks' => $networks,
      '#language_selectors' => $language_selectors,
    ]);

    return $post_groups;
  }

  /**
   * {@inheritdoc}
   *
   * Social posts grouped by social network with language selectors.
   *
   * @throws \Symfony\Component\DependencyInjection\Exception\InvalidArgumentException
   *   Occurs when the arguments are not coherent or not supported.
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $post_groups = [];
    /** @var \Drupal\ino_pt_social_post\Plugin\Field\FieldType\socialPost $item */
    foreach ($items as $item) {
      /** @var \Drupal\ino_pt_social_post\SocialPostFormatter\SocialPostFormatterInterface $sp */
      try {
        $sp = $this->socialPostFormatter::createInstance($item, $langcode);
        $social_network = $sp->getNetworkName();
        $post_lang = $sp->getLanguage();

        if (!isset($post_groups[$social_network])) {
          $post_groups[$social_network] = [
            '#theme' => 'ino_pt_social_posts',
            '#attributes' => [
              'class' => [
                'social-posts',
                'social-posts-' . $social_network,
              ],
              'data-network' => [$social_network],
            ],
            '#ino_pt_social_posts_info' => [
              'network' => $social_network,
              'langcodes' => [],
            ],
          ];
        }
        $post_groups[$social_network]['#ino_pt_social_posts_info']['langcodes'][$post_lang] = $post_lang;
        $post_groups[$social_network][] = $sp->getPost();
      }
      catch (InvalidArgumentException $e) {
        $this->logger('ino_pt_social_post')->notice($e->getMessage());
      }
    }

    foreach (array_keys($post_groups) as $social_network) {
      $is_multiple = count($post_groups[$social_network]) > 2;
      $post_groups[$social_network]['#attributes']['class'][] = $is_multiple ? 'social-posts-multi' : 'social-posts-single';
    }

    // A renderable array for $items, as an array of child elements.
    return array_values($post_groups);
  }

  /**
   * Runs all the enabled filters on a piece of text. Remove html tags.
   *
   * @param string $text
   *   The text to be filtered.
   * @param string $format_id
   *   The machine name of the filter format to be used to filter the
   *   text.
   *
   * @return string
   *   The filtered text.
   */
  protected function getShareMessage(string $text, string $format_id) {
    $text = check_markup($text, $format_id);
    $text = strip_tags($text);

    return $text;
  }

}
