<?php
/**
 * @file
 * Contains \Drupal\ino_pt_social_post\Plugin\Field\FieldWidget\SocialPostWidget.
 */

namespace Drupal\ino_pt_social_post\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\languagefield\Entity\CustomLanguageManager;


/**
 * Plugin implementation of the 'social post' widget.
 *
 * @FieldWidget (
 *   id = "social-post",
 *   label = @Translation("Social post"),
 *   field_types = {
 *     "social-post"
 *   }
 * )
 */
class SocialPostWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $languages = $this->getPossibleOptions();

    $element['title'] = array(
      '#type' => 'textfield',
      '#title' => t('Admin title'),
      '#default_value' => $items[$delta]->title ?? NULL,
    );
    $element['value'] = array(
      '#type' => 'select',
      '#title' => t('Social network'),
      '#default_value' => $items[$delta]->value ?? NULL,
      '#field_prefix' => '<span class="form-required"></span>',
      '#empty_option' => $this->t('- None -'),
      '#options' => $this->getFieldSetting('allowed_values'),
    );
    $element['content'] = array(
      '#type' => 'text_format',
      '#title' => t('Content'),
      '#field_prefix' => '<span class="form-required"></span>',
      '#format' => 'ino_format_social_post',
      '#allowed_formats' => ['ino_format_social_post'],
      '#default_value' => $items[$delta]->content ?? NULL,
    );
    $element['language'] = array(
      '#type' => 'select',
      '#title' => t('Language'),
      '#field_prefix' => '<span class="form-required"></span>',
      '#empty_option' => $this->t('- None -'),
      '#options' => $languages,
      '#default_value' => $items[$delta]->language ?? NULL,
    );

    $element['#element_validate'] = [[static::class, 'validateItems']];

    return $element;
  }

  /**
   * Validates that newly selected items can be added to the widget.
   *
   * Making an invalid selection from the view should not be possible, but we
   * still validate in case other selection methods (ex: upload) are valid.
   *
   * @param array $element
   *   The form element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public static function validateItems($element, FormStateInterface $form_state) {
    $value = $element['value']['#value'];
    $content = $element['content']['value']['#value'];
    $language = $element['language']['#value'];

    if (!(empty($value) && empty($content) && empty($language))) {
      if (empty($value)) {
        $form_state->setError($element['value'], \Drupal::translation()->translate('@name field is required.', ['@name' => $element['value']['#title']]));
      }

      if (empty($content)) {
        $form_state->setError($element['content'], \Drupal::translation()->translate('@name field is required.', ['@name' => $element['content']['#title']]));
      }

      if (empty($language)) {
        $form_state->setError($element['language'], \Drupal::translation()->translate('@name field is required.', ['@name' => $element['language']['#title']]));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    foreach($values as $key => $value) {
      $values[$key]['content'] = $value['content']['value'];
    }
    return $values;
  }

  /**
   * {@inheritdoc}
   *
   * @param $format
   *   Extra parameter for formatting options.
   */
  public function getPossibleOptions(AccountInterface $account = NULL, $format = 'en') {
    $select_options = [];

    // No need to cache this data. It is a hardcoded list.
    $languages = \Drupal::languageManager()->getStandardLanguageList();
    // Add the custom languages to the list.
    $languages += CustomLanguageManager::getCustomLanguageList();

    // Format the array to Options format.
    foreach ($languages as $langcode => $language_names) {
      $language_name = '';
      switch ($format) {
        case 'en':
          $language_name .= $this->t($language_names[0]);
          break;
        case 'loc':
          $language_name .= $language_names[1];
          break;
        case 'both':
          $language_name .= $this->t($language_names[0]);
          if (mb_strlen($language_names[1])) {
            $language_name .= ' (' . $language_names[1] . ')';
          }
          $language_name .= ' [' . $langcode . ']';
          break;
      }

      $select_options[$langcode] = $language_name;
    }

    asort($select_options);

    return $select_options;
  }

}