<?php

namespace Drupal\ino_pt_social_post\SocialPostFormatter;

use Drupal\ino_pt_social_post\Plugin\Field\FieldType\socialPost;
use Drupal\Core\Link;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Class SocialPostFormatterBase.
 *
 * @package Drupal\ino_pt_social_post\SocialPostFormatter
 */
abstract class SocialPostFormatterBase implements SocialPostFormatterInterface {
  use StringTranslationTrait;

  /**
   * Field item.
   *
   * @var \Drupal\ino_pt_social_post\Plugin\Field\FieldType\socialPost
   */
  private $item;

  /**
   * Langcode.
   *
   * @var \Drupal\Core\Entity\Query\QueryInterface
   */
  private $langcode;

  /**
   * BaseFilterDecorator constructor.
   *
   * @param socialPost $item
   *   The field item.
   * @param string $langcode
   *   The language code.
   */
  public function __construct(socialPost $item, string $langcode) {
    $this->item = $item;
    $this->langcode = $langcode;

  }

  /**
   * {@inheritdoc}
   */
  public function getLanguage(): string {
    return $this->item->getValue()['language'];
  }

  /**
   * {@inheritdoc}
   */
  public function getNetworkName(): string {
    return $this->item->getValue()['value'];
  }

  /**
   * {@inheritdoc}
   */
  public function getContent(): string {
    if (!empty($this->item->getValue()['content'])) {
      $content = $this->item->getValue()['content'];
    }
    else {
      $content = 'Content Missing';
    }

    return $content;
  }

  /**
   * {@inheritdoc}
   */
  public function getPost(): array {
    $social_network = $this->getNetworkName();
    $post_lang = $this->getLanguage();

    $post = [
      '#type' => 'container',
      '#attributes' => [
        'class' => [
          'social-post',
          'social-post-' . $social_network,
          'social-post-lang-' . $post_lang,
        ],
        'data-network' => [$social_network],
        'data-language' => [$post_lang],
      ],
      // Create a cache tag entry for the referenced entity. In the case
      // that the referenced entity is deleted, the cache for referring
      // entities must be cleared.
      '#cache' => [
        'tags' => $this->getParentCacheTags(),
      ],
      'message' => $this->getMessage(),
      'button' => $this->getButton(),
    ];

    return $post;
  }

  /**
   * The cache tags associated with this object.
   *
   * When this object is modified, these cache tags will be invalidated.
   *
   * @return string[]
   *   A set of cache tags.
   */
  protected function getParentCacheTags(): array {
    /** @var \Drupal\paragraphs\Entity\Paragraph $paragraph */
    $paragraph = $this->item->getEntity();
    return $paragraph->getCacheTags();
  }

  /**
   * Get content format.
   *
   * @param string $format
   *   Text formatter name.
   *
   * @return string
   *   Social media text formatter name.
   */
  abstract protected function getContentFormat(string $format): string;

  /**
   * Get social media message.
   *
   * @return array
   *   A message render array.
   */
  protected function getMessage(): array {
    $content = $this->getContent();

    $message = [
      '#type' => 'processed_text',
      '#text' => $content,
      '#format' => $this->getContentFormat('ino_format_social_post'),
    ];

    return $message;
  }

  /**
   * Get button link.
   */
  abstract protected function getButtonLink(): Link;

  /**
   * Get social post link button.
   *
   * @return array
   *   A button render array.
   */
  protected function getButton(): array {
    $link = $this->getButtonLink();

    $button = $link->toRenderable();
    $button['#attributes']['class'] = [
      'button',
      'link-social-post',
      'link-social-post-' . $this->getNetworkName(),
      'link-social-post-lang-' . $this->getLanguage(),
    ];
    $button['#attributes']['target'] = '_blank';

    return $button;
  }

  /**
   * Runs all the enabled filters on a piece of text. Remove html tags.
   *
   * @param string $text
   *   The text to be filtered.
   *
   * @param string $format
   *
   * @return string
   *   The filtered text.
   */
  protected function cleanMessage(string $text, string $format): string {
    $text = check_markup($text, $format);
    $text = strip_tags($text);

    return trim($text);
  }

}
